package cn.com.qmth.examcloud.core.basic.api.controller;

import javax.servlet.http.HttpServletRequest;

import org.apache.commons.lang3.RandomUtils;
import org.apache.commons.lang3.StringUtils;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.transaction.annotation.Transactional;
import org.springframework.web.bind.annotation.PostMapping;
import org.springframework.web.bind.annotation.RequestBody;
import org.springframework.web.bind.annotation.RequestMapping;
import org.springframework.web.bind.annotation.RequestParam;
import org.springframework.web.bind.annotation.RestController;

import cn.com.qmth.examcloud.commons.base.exception.StatusException;
import cn.com.qmth.examcloud.commons.base.util.PropertiesUtil;
import cn.com.qmth.examcloud.commons.web.security.bean.User;
import cn.com.qmth.examcloud.commons.web.support.ControllerSupport;
import cn.com.qmth.examcloud.core.basic.base.constants.PropKeys;
import cn.com.qmth.examcloud.core.basic.dao.StudentRepo;
import cn.com.qmth.examcloud.core.basic.dao.entity.StudentEntity;
import cn.com.qmth.examcloud.core.basic.service.AuthService;
import cn.com.qmth.examcloud.core.basic.service.bean.LoginInfo;
import cn.com.qmth.examcloud.exchange.inner.api.SendSmsCloudService;
import cn.com.qmth.examcloud.exchange.inner.api.request.CheckSmsCodeReq;
import cn.com.qmth.examcloud.exchange.inner.api.request.SendSmsReq;
import cn.com.qmth.examcloud.exchange.inner.api.response.CheckIdentifyingCodeResp;
import io.swagger.annotations.ApiOperation;

/**
 * 鉴权
 *
 * @author WANGWEI
 * @date 2018年5月25日
 * @Copyright (c) 2018-? http://qmth.com.cn All Rights Reserved.
 */
@Transactional
@RestController
@RequestMapping("${$rmp.ctr.basic}/" + "auth")
public class AuthController extends ControllerSupport {

	@Autowired
	AuthService authService;

	@Autowired
	SendSmsCloudService sendSmsCloudService;

	@Autowired
	StudentRepo studentRepo;

	@ApiOperation(value = "登入", notes = "")
	@PostMapping("login")
	public User login(@RequestBody LoginInfo loginInfo, HttpServletRequest request) {
		String realIp = request.getHeader("x-forwarded-for");
		if (StringUtils.isBlank(realIp)) {
			realIp = request.getHeader("x-real-ip");
		}
		if (StringUtils.isNotBlank(realIp)) {
			loginInfo.setClientIp(realIp);
		} else {
			loginInfo.setClientIp(null);
		}
		User user = authService.login(loginInfo);
		return user;
	}

	@ApiOperation(value = "登出", notes = "")
	@PostMapping("logout")
	public void logout() {
		User user = getAccessUser();
		if (null == user) {
			throw new StatusException("B-001001", "用户已登出");
		}
		authService.logout(user);
	}

	@ApiOperation(value = "获取登录用户", notes = "")
	@PostMapping("getLoginUser")
	public User getLoginUser(@RequestParam String key, @RequestParam String token) {

		if (StringUtils.isBlank(key)) {
			throw new StatusException("B-001009", "key is blank");
		}
		if (StringUtils.isBlank(token)) {
			throw new StatusException("B-001010", "token is blank");
		}

		return authService.getLoginUser(key, token);
	}

	@ApiOperation(value = "第三方机构接入", notes = "")
	@PostMapping("/thirdPartyAccess")
	public User thirdPartyAccess(@RequestParam Long orgId, @RequestParam String loginName,
			@RequestParam String appId, @RequestParam String timestamp,
			@RequestParam String token) {
		return authService.thirdPartyAccess(orgId, loginName, appId, timestamp, token);
	}

	@ApiOperation(value = "发送验证码", notes = "")
	@PostMapping("/sendVerificationCode")
	public void sendVerificationCode(@RequestParam String phone) {
		SendSmsReq req = new SendSmsReq();
		req.setPhone(phone);
		int code = 1000 + RandomUtils.nextInt(1, 9999);
		req.setCode(String.valueOf(code));

		int smsEffectivetime = PropertiesUtil.getInt(PropKeys.SEND_VERIFICATION_CODE_EFFECTIVE_TIME,
				120);
		int smsIntervalSeconds = PropertiesUtil
				.getInt(PropKeys.SEND_VERIFICATION_CODE_INTERVAL_SECONDS, 60);

		String smsSign = PropertiesUtil.getString(PropKeys.SEND_VERIFICATION_CODE_SIGN);
		String smsTemplatecode = PropertiesUtil
				.getString(PropKeys.SEND_VERIFICATION_CODE_TEMPLATE_CODE);

		if (StringUtils.isBlank(smsSign)) {
			throw new StatusException("B-001060", "签名未配置");
		}
		if (StringUtils.isBlank(smsTemplatecode)) {
			throw new StatusException("B-001061", "模板未配置");
		}

		req.setEffectiveTime(smsEffectivetime);
		req.setIntervalSeconds(smsIntervalSeconds);
		req.setSign(smsSign);
		req.setTemplateCode(smsTemplatecode);
		sendSmsCloudService.sendIdentifyingCode(req);
	}

	@ApiOperation(value = "绑定手机号", notes = "")
	@PostMapping("/bindSecurityPhone")
	public void bindSecurityPhone(@RequestParam String phone,
			@RequestParam String verificationCode) {
		CheckSmsCodeReq req = new CheckSmsCodeReq();
		req.setCode(verificationCode);
		req.setPhone(phone);
		CheckIdentifyingCodeResp resp = sendSmsCloudService.checkIdentifyingCode(req);
		if (!resp.getSuccess()) {
			throw new StatusException("B-001050", "验证码错误");
		}

		User accessUser = getAccessUser();
		Long studentId = accessUser.getUserId();

		StudentEntity sb = studentRepo.findBySecurityPhone(phone);
		if (null != sb) {
			if (!sb.getId().equals(studentId)) {
				throw new StatusException("B-001052", "手机号已被其他用户绑定,请更换手机号");
			} else {
				throw new StatusException("B-001053", "手机号已绑定成功,无需重复绑定");
			}
		}

		StudentEntity s = studentRepo.findOne(studentId);
		s.setSecurityPhone(phone);
		studentRepo.saveAndFlush(s);
	}

}