package cn.com.qmth.examcloud.core.basic.api.controller;

import java.io.File;
import java.util.ArrayList;
import java.util.List;
import java.util.Map;
import java.util.stream.Collectors;
import java.util.stream.Stream;

import javax.persistence.criteria.Predicate;
import javax.persistence.criteria.Root;
import javax.persistence.criteria.Subquery;
import javax.servlet.http.HttpServletResponse;

import org.apache.commons.collections.CollectionUtils;
import org.apache.commons.fileupload.disk.DiskFileItem;
import org.apache.commons.io.FileUtils;
import org.apache.commons.lang3.StringUtils;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.data.domain.Page;
import org.springframework.data.domain.PageRequest;
import org.springframework.data.domain.Sort;
import org.springframework.data.domain.Sort.Direction;
import org.springframework.data.jpa.domain.Specification;
import org.springframework.transaction.annotation.Transactional;
import org.springframework.web.bind.annotation.DeleteMapping;
import org.springframework.web.bind.annotation.GetMapping;
import org.springframework.web.bind.annotation.PathVariable;
import org.springframework.web.bind.annotation.PostMapping;
import org.springframework.web.bind.annotation.PutMapping;
import org.springframework.web.bind.annotation.RequestBody;
import org.springframework.web.bind.annotation.RequestMapping;
import org.springframework.web.bind.annotation.RequestParam;
import org.springframework.web.bind.annotation.RestController;
import org.springframework.web.multipart.commons.CommonsMultipartFile;

import com.google.common.collect.Lists;
import com.google.common.collect.Maps;

import cn.com.qmth.examcloud.commons.base.exception.StatusException;
import cn.com.qmth.examcloud.commons.base.helpers.poi.ExcelWriter;
import cn.com.qmth.examcloud.commons.base.util.PathUtil;
import cn.com.qmth.examcloud.commons.web.security.bean.User;
import cn.com.qmth.examcloud.commons.web.support.ControllerSupport;
import cn.com.qmth.examcloud.commons.web.support.SystemConfig;
import cn.com.qmth.examcloud.core.basic.api.CourseCloudService;
import cn.com.qmth.examcloud.core.basic.api.controller.bean.CourseDomain;
import cn.com.qmth.examcloud.core.basic.dao.CourseRepo;
import cn.com.qmth.examcloud.core.basic.dao.entity.CourseEntity;
import cn.com.qmth.examcloud.core.basic.dao.entity.CourseSpeciatlyRelationEntity;
import cn.com.qmth.examcloud.core.basic.dao.enums.CourseLevel;
import cn.com.qmth.examcloud.core.basic.service.CourseService;
import cn.com.qmth.examcloud.core.basic.service.bean.CourseInfo;
import io.swagger.annotations.ApiOperation;

/**
 * 课程服务API Created by songyue on 17/1/14.
 */
@Transactional
@RestController
@RequestMapping("${$rmp.ctr.basic}/course")
public class CourseController extends ControllerSupport {

	@Autowired
	CourseRepo courseRepo;

	@Autowired
	CourseService courseService;

	@Autowired
	CourseCloudService courseCloudService;

	private static final String[] EXCEL_HEADER = new String[]{"课程名称", "课程代码", "层次(ZSB,GQZ,ALL)"};

	/**
	 * 方法注释
	 *
	 * @author WANGWEI
	 * @param curPage
	 * @param pageSize
	 * @param name
	 * @param code
	 * @param level
	 * @param enable
	 * @param specialtyId
	 * @return
	 */
	@ApiOperation(value = "分页查询课程")
	@GetMapping("coursePage/{curPage}/{pageSize}")
	public Page<CourseEntity> getCoursePage(@PathVariable Integer curPage,
			@PathVariable Integer pageSize, @RequestParam(required = false) String name,
			@RequestParam(required = false) String code,
			@RequestParam(required = false) String level,
			@RequestParam(required = false) Boolean enable,
			@RequestParam(required = false) Long specialtyId) {

		User accessUser = getAccessUser();

		Specification<CourseEntity> specification = (root, query, cb) -> {
			List<Predicate> predicates = new ArrayList<>();

			predicates.add(cb.equal(root.get("rootOrgId"), accessUser.getRootOrgId()));

			if (StringUtils.isNotBlank(name)) {
				predicates.add(cb.like(root.get("name"), toSqlSearchPattern(name)));
			}
			if (StringUtils.isNotBlank(code)) {
				predicates.add(cb.like(root.get("code"), toSqlSearchPattern(code)));
			}
			if (StringUtils.isNotBlank(level)) {
				predicates.add(cb.equal(root.get("level"), CourseLevel.valueOf(level)));
			}
			if (null != enable) {
				predicates.add(cb.equal(root.get("enable"), enable));
			}

			if (null != specialtyId) {
				Subquery<CourseSpeciatlyRelationEntity> subquery = query
						.subquery(CourseSpeciatlyRelationEntity.class);
				Root<CourseSpeciatlyRelationEntity> subRoot = subquery
						.from(CourseSpeciatlyRelationEntity.class);
				subquery.select(subRoot.get("courseId"));
				Predicate p1 = cb.equal(subRoot.get("specialtyId"), specialtyId);
				Predicate p2 = cb.equal(subRoot.get("courseId"), root.get("id"));
				subquery.where(cb.and(p1, p2));
				predicates.add(cb.exists(subquery));
			}

			return cb.and(predicates.toArray(new Predicate[predicates.size()]));
		};

		PageRequest pageRequest = new PageRequest(curPage, pageSize,
				new Sort(Direction.DESC, "updateTime"));

		Page<CourseEntity> page = courseRepo.findAll(specification, pageRequest);
		return page;
	}

	/**
	 * 方法注释
	 *
	 * @author WANGWEI
	 * @param name
	 * @param code
	 * @param level
	 * @param enable
	 * @param specialtyId
	 * @return
	 */
	@ApiOperation(value = "查询课程")
	@GetMapping("query")
	public List<CourseEntity> query(@RequestParam(required = false) String name,
			@RequestParam(required = false) String code,
			@RequestParam(required = false) String level,
			@RequestParam(required = false) Boolean enable,
			@RequestParam(required = false) Long specialtyId) {

		User accessUser = getAccessUser();
		Long rootOrgId = accessUser.getRootOrgId();

		Specification<CourseEntity> specification = (root, query, cb) -> {
			List<Predicate> predicates = new ArrayList<>();

			predicates.add(cb.equal(root.get("rootOrgId"), rootOrgId));

			if (StringUtils.isNotBlank(name)) {
				predicates.add(cb.like(root.get("name"), toSqlSearchPattern(name)));
			}
			if (StringUtils.isNotBlank(code)) {
				predicates.add(cb.like(root.get("code"), toSqlSearchPattern(code)));
			}
			if (StringUtils.isNotBlank(level)) {
				predicates.add(cb.equal(root.get("level"), toSqlSearchPattern(level)));
			}
			if (null != enable) {
				predicates.add(cb.equal(root.get("enable"), enable));
			}

			if (null != specialtyId) {
				Subquery<CourseSpeciatlyRelationEntity> subquery = query
						.subquery(CourseSpeciatlyRelationEntity.class);
				Root<CourseSpeciatlyRelationEntity> subRoot = subquery
						.from(CourseSpeciatlyRelationEntity.class);
				subquery.select(subRoot.get("courseId"));
				Predicate p1 = cb.equal(subRoot.get("specialtyId"), specialtyId);
				Predicate p2 = cb.equal(subRoot.get("courseId"), root.get("id"));
				subquery.where(cb.and(p1, p2));
				predicates.add(cb.exists(subquery));
			}

			return cb.and(predicates.toArray(new Predicate[predicates.size()]));
		};

		// 过载保护
		long total = courseRepo.count(specification);
		if (total > 1000) {
			List<CourseEntity> list = Lists.newArrayList();
			return list;
		}

		List<CourseEntity> list = courseRepo.findAll(specification);
		return list;
	}

	/**
	 * 方法注释
	 *
	 * @author WANGWEI
	 * @param id
	 * @return
	 */
	@ApiOperation(value = "按ID查询课程", notes = "ID查询")
	@GetMapping("{id}")
	public CourseEntity getCourseById(@PathVariable Long id) {
		CourseEntity course = courseRepo.findOne(id);
		if (null == course) {
			throw new StatusException("B-540001", "课程不存在");
		}

		Long rootOrgId = course.getRootOrgId();
		validateRootOrgIsolation(rootOrgId);
		return course;
	}

	/**
	 * 方法注释
	 *
	 * @author WANGWEI
	 * @param rootOrgId
	 * @param code
	 * @return
	 */
	@ApiOperation(value = "按code查询课程", notes = "code查询")
	@GetMapping("byCode")
	public CourseEntity getCourseByCode(@RequestParam Long rootOrgId, @RequestParam String code) {
		validateRootOrgIsolation(rootOrgId);

		CourseEntity course = courseRepo.findByRootOrgIdAndCode(rootOrgId, code);
		if (null == course) {
			throw new StatusException("B-540001", "课程不存在");
		}

		return course;
	}

	/**
	 * 修正
	 *
	 * @author WANGWEI
	 * @param domain
	 * @return
	 */
	@ApiOperation(value = "新增课程", notes = "新增")
	@PostMapping
	public Long addCourse(@RequestBody CourseDomain domain) {
		trim(domain, true);

		User accessUser = getAccessUser();
		Long rootOrgId = accessUser.getRootOrgId();

		String code = domain.getCode();
		if (StringUtils.isBlank(code)) {
			throw new StatusException("B-620001", "code is blank");
		}
		CourseEntity course = courseRepo.findByRootOrgIdAndCode(rootOrgId, code);
		if (null != course) {
			throw new StatusException("B-620002", "课程编码已被占用");
		}

		CourseInfo info = new CourseInfo();
		info.setRootOrgId(rootOrgId);
		info.setCode(domain.getCode());
		info.setEnable(domain.getEnable());
		info.setId(domain.getId());
		info.setLevel(domain.getLevel());
		info.setName(domain.getName());

		CourseEntity saved = courseService.saveCourse(info);
		return saved.getId();
	}

	/**
	 * 修正
	 *
	 * @author WANGWEI
	 * @param domain
	 * @return
	 */
	@ApiOperation(value = "更新课程", notes = "更新")
	@PutMapping
	public Long updateCourse(@RequestBody CourseDomain domain) {
		trim(domain, true);

		User accessUser = getAccessUser();
		Long rootOrgId = accessUser.getRootOrgId();

		CourseInfo info = new CourseInfo();
		info.setRootOrgId(rootOrgId);
		info.setCode(domain.getCode());
		info.setEnable(domain.getEnable());
		info.setId(domain.getId());
		info.setLevel(domain.getLevel());
		info.setName(domain.getName());

		CourseEntity saved = courseService.saveCourse(info);
		return saved.getId();
	}

	@ApiOperation(value = "删除课程", notes = "删除")
	@DeleteMapping("{ids}")
	public void deleteCourse(@PathVariable String ids) {
		List<Long> courseIds = Stream.of(ids.split(",")).map(s -> Long.parseLong(s.trim()))
				.collect(Collectors.toList());
		for (Long courseId : courseIds) {
			CourseEntity one = courseRepo.findOne(courseId);
			if (null == one) {
				continue;
			}
			validateRootOrgIsolation(one.getRootOrgId());
			courseRepo.delete(one);
		}
	}

	@ApiOperation(value = "下载导入模板", notes = "下载导入模板")
	@GetMapping("importTemplate")
	public void getImportTemplate(HttpServletResponse response) {
		String resoucePath = PathUtil.getResoucePath("templates/courseImportTemplate.xlsx");
		exportFile("课程导入模板.xlsx", new File(resoucePath));
	}

	@ApiOperation(value = "导入课程", notes = "导入")
	@PostMapping("import")
	public Map<String, Object> importCourse(@RequestParam CommonsMultipartFile file) {
		DiskFileItem item = (DiskFileItem) file.getFileItem();
		File storeLocation = item.getStoreLocation();
		List<Map<String, Object>> failRecords = courseService.importCourse(getRootOrgId(),
				storeLocation);
		Map<String, Object> map = Maps.newHashMap();
		map.put("hasError", CollectionUtils.isNotEmpty(failRecords));
		map.put("failRecords", failRecords);
		return map;
	}

	@ApiOperation(value = "导出课程", notes = "导出")
	@GetMapping("export")
	public void export(@RequestParam(required = false) String name,
			@RequestParam(required = false) String code,
			@RequestParam(required = false) String level,
			@RequestParam(required = false) Boolean enable,
			@RequestParam(required = false) Long specialtyId) {

		User accessUser = getAccessUser();

		Specification<CourseEntity> specification = (root, query, cb) -> {
			List<Predicate> predicates = new ArrayList<>();

			predicates.add(cb.equal(root.get("rootOrgId"), accessUser.getRootOrgId()));

			if (StringUtils.isNotBlank(name)) {
				predicates.add(cb.like(root.get("name"), toSqlSearchPattern(name)));
			}
			if (StringUtils.isNotBlank(code)) {
				predicates.add(cb.like(root.get("code"), toSqlSearchPattern(code)));
			}
			if (StringUtils.isNotBlank(level)) {
				predicates.add(cb.equal(root.get("level"), CourseLevel.valueOf(level)));
			}
			if (null != enable) {
				predicates.add(cb.equal(root.get("enable"), enable));
			}

			if (null != specialtyId) {
				Subquery<CourseSpeciatlyRelationEntity> subquery = query
						.subquery(CourseSpeciatlyRelationEntity.class);
				Root<CourseSpeciatlyRelationEntity> subRoot = subquery
						.from(CourseSpeciatlyRelationEntity.class);
				subquery.select(subRoot.get("courseId"));
				Predicate p1 = cb.equal(subRoot.get("specialtyId"), specialtyId);
				Predicate p2 = cb.equal(subRoot.get("courseId"), root.get("id"));
				subquery.where(cb.and(p1, p2));
				predicates.add(cb.exists(subquery));
			}

			return cb.and(predicates.toArray(new Predicate[predicates.size()]));
		};

		long count = courseRepo.count(specification);
		if (100000 < count) {
			throw new StatusException("B-620200", "数据量过大,无法导出");
		}

		List<CourseEntity> list = courseRepo.findAll(specification);

		List<Object[]> datas = Lists.newArrayList();

		for (CourseEntity cur : list) {
			datas.add(new Object[]{cur.getName(), cur.getCode(), cur.getLevel().name()});
		}

		String filePath = SystemConfig.getTempDataDir() + File.separator
				+ System.currentTimeMillis() + ".xlsx";
		File file = new File(filePath);

		ExcelWriter.write(EXCEL_HEADER, new Class[]{String.class, String.class, String.class},
				datas, new File(filePath));

		exportFile("课程列表-" + getRootOrgId() + ".xlsx", file);

		FileUtils.deleteQuietly(file);
	}

	/**
	 * 方法注释
	 *
	 * @author WANGWEI
	 * @param ids
	 * @return
	 */
	@ApiOperation(value = "禁用课程", notes = "禁用")
	@PutMapping("disable/{ids}")
	public List<Long> disableCourse(@PathVariable String ids) {
		List<Long> courseIds = Stream.of(ids.split(",")).map(s -> Long.parseLong(s.trim()))
				.collect(Collectors.toList());

		for (Long courseId : courseIds) {
			CourseEntity course = courseRepo.findOne(courseId);
			course.setEnable(false);
			courseRepo.save(course);
		}

		return courseIds;
	}

	/**
	 * 方法注释
	 *
	 * @author WANGWEI
	 * @param ids
	 * @return
	 */
	@ApiOperation(value = "启用课程", notes = "启用")
	@PutMapping("enable/{ids}")
	public List<Long> enableCourse(@PathVariable String ids) {
		List<Long> courseIds = Stream.of(ids.split(",")).map(s -> Long.parseLong(s.trim()))
				.collect(Collectors.toList());

		for (Long courseId : courseIds) {
			CourseEntity course = courseRepo.findOne(courseId);
			course.setEnable(true);
			courseRepo.save(course);
		}

		return courseIds;
	}
}
