package cn.com.qmth.examcloud.core.basic.api.controller;

import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.transaction.annotation.Transactional;
import org.springframework.web.bind.annotation.DeleteMapping;
import org.springframework.web.bind.annotation.PostMapping;
import org.springframework.web.bind.annotation.RequestMapping;
import org.springframework.web.bind.annotation.RequestParam;
import org.springframework.web.bind.annotation.RestController;

import cn.com.qmth.examcloud.commons.base.exception.StatusException;
import cn.com.qmth.examcloud.commons.web.security.bean.User;
import cn.com.qmth.examcloud.commons.web.support.ControllerSupport;
import cn.com.qmth.examcloud.core.basic.dao.CourseRepo;
import cn.com.qmth.examcloud.core.basic.dao.CourseSpeciatlyRelationRepo;
import cn.com.qmth.examcloud.core.basic.dao.SpecialtyRepo;
import cn.com.qmth.examcloud.core.basic.dao.entity.CourseEntity;
import cn.com.qmth.examcloud.core.basic.dao.entity.CourseSpeciatlyRelationEntity;
import cn.com.qmth.examcloud.core.basic.dao.entity.CourseSpeciatlyRelationPK;
import cn.com.qmth.examcloud.core.basic.dao.entity.SpecialtyEntity;
import io.swagger.annotations.ApiOperation;

/**
 * @Description 课程专业关联API
 * @author Administrator
 * @Date 2017年8月8号
 */
@Transactional
@RestController
@RequestMapping("${$rmp.ctr.basic}/courseSpeciatlyRelation")
public class CourseSpeciatlyRelationController extends ControllerSupport {

	@Autowired
	CourseSpeciatlyRelationRepo courseSpeciatlyRelationRepo;

	@Autowired
	CourseRepo courseRepo;

	@Autowired
	SpecialtyRepo specialtyRepo;

	@ApiOperation(value = "新增关联课程专业")
	@PostMapping("add")
	public CourseSpeciatlyRelationEntity add(@RequestParam(required = true) Long courseId,
			@RequestParam(required = true) Long specialtyId) {
		User accessUser = getAccessUser();

		if (null == courseId) {
			throw new StatusException("B-320001", "courseId is null");
		}

		if (null == specialtyId) {
			throw new StatusException("B-320002", "specialtyId is null");
		}

		CourseEntity courseEntity = courseRepo.findOne(courseId);
		if (null == courseEntity) {
			throw new StatusException("B-320001", "courseId is wrong");
		}

		this.validateRootOrgIsolation(courseEntity.getRootOrgId());

		SpecialtyEntity specialtyEntity = specialtyRepo.findOne(specialtyId);
		if (null == specialtyEntity) {
			throw new StatusException("B-320001", "specialtyId is wrong");
		}

		this.validateRootOrgIsolation(specialtyEntity.getRootOrgId());

		CourseSpeciatlyRelationPK pk = new CourseSpeciatlyRelationPK();
		pk.setCourseId(courseId);
		pk.setSpecialtyId(specialtyId);

		CourseSpeciatlyRelationEntity one = courseSpeciatlyRelationRepo.findOne(pk);
		if (null != one) {
			throw new StatusException("B-320003", "课程专业已关联");
		}

		one = new CourseSpeciatlyRelationEntity();
		one.setCourseId(courseId);
		one.setSpecialtyId(specialtyId);
		one.setCreator(accessUser.getUserId());

		CourseSpeciatlyRelationEntity saved = courseSpeciatlyRelationRepo.save(one);

		return saved;
	}

	@ApiOperation(value = "删除关联课程专业")
	@DeleteMapping("delete")
	public CourseSpeciatlyRelationEntity delete(@RequestParam(required = true) Long courseId,
			@RequestParam(required = true) Long specialtyId) {

		if (null == courseId) {
			throw new StatusException("B-320001", "courseId is null");
		}

		if (null == specialtyId) {
			throw new StatusException("B-320002", "specialtyId is null");
		}

		CourseEntity courseEntity = courseRepo.findOne(courseId);
		if (null == courseEntity) {
			throw new StatusException("B-320001", "courseId is wrong");
		}

		this.validateRootOrgIsolation(courseEntity.getRootOrgId());

		SpecialtyEntity specialtyEntity = specialtyRepo.findOne(specialtyId);
		if (null == specialtyEntity) {
			throw new StatusException("B-320001", "specialtyId is wrong");
		}

		this.validateRootOrgIsolation(specialtyEntity.getRootOrgId());

		CourseSpeciatlyRelationPK pk = new CourseSpeciatlyRelationPK();
		pk.setCourseId(courseId);
		pk.setSpecialtyId(specialtyId);

		CourseSpeciatlyRelationEntity one = courseSpeciatlyRelationRepo.findOne(pk);
		if (null == one) {
			throw new StatusException("B-320003", "课程专业未关联");
		}

		courseSpeciatlyRelationRepo.delete(one);

		return one;
	}

}
