package cn.com.qmth.examcloud.core.basic.api.controller;

import java.util.List;

import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.transaction.annotation.Transactional;
import org.springframework.web.bind.annotation.GetMapping;
import org.springframework.web.bind.annotation.PathVariable;
import org.springframework.web.bind.annotation.PostMapping;
import org.springframework.web.bind.annotation.RequestBody;
import org.springframework.web.bind.annotation.RequestMapping;
import org.springframework.web.bind.annotation.RestController;

import cn.com.qmth.examcloud.commons.base.exception.StatusException;
import cn.com.qmth.examcloud.commons.web.support.ControllerSupport;
import cn.com.qmth.examcloud.core.basic.api.controller.bean.StudentFaceDomain;
import cn.com.qmth.examcloud.core.basic.dao.StudentFaceRepo;
import cn.com.qmth.examcloud.core.basic.dao.StudentRepo;
import cn.com.qmth.examcloud.core.basic.dao.entity.FacesetEntity;
import cn.com.qmth.examcloud.core.basic.dao.entity.StudentFaceEntity;
import cn.com.qmth.examcloud.core.basic.service.FaceService;
import cn.com.qmth.examcloud.core.basic.service.bean.StudentFaceInfo;
import io.swagger.annotations.ApiOperation;

/**
 * 类注释
 *
 * @author WANGWEI
 * @date 2018年9月4日
 * @Copyright (c) 2018-? http://qmth.com.cn All Rights Reserved.
 */
@Transactional
@RestController
@RequestMapping("${$rmp.ctr.basic}/face")
public class FaceController extends ControllerSupport {

	@Autowired
	StudentFaceRepo studentFaceRepo;

	@Autowired
	StudentRepo studentRepo;

	@Autowired
	FaceService studentFaceService;

	@ApiOperation(value = "保存学生人脸数据")
	@PostMapping("saveStudentFace")
	public StudentFaceEntity saveStudentFace(@RequestBody StudentFaceDomain req) {
		StudentFaceInfo info = new StudentFaceInfo();
		info.setFacesetToken(req.getFacesetToken());
		info.setFaceToken(req.getFaceToken());
		info.setOperator(req.getOperator());
		info.setPhotoName(req.getPhotoName());
		info.setRootOrgId(req.getRootOrgId());
		info.setStudentId(req.getStudentId());
		info.setFaceCount(req.getFaceCount());

		StudentFaceEntity saved = studentFaceService.saveStudentFace(info);
		return saved;
	}

	@ApiOperation(value = "获取可用的faceset集合")
	@GetMapping("getUsableFacesetList")
	public List<FacesetEntity> getUsableFacesetList() {

		List<FacesetEntity> facesetList = studentFaceService.getUsableFacesetList();

		return facesetList;
	}

	@ApiOperation(value = "获取可用的faceset集合")
	@GetMapping("studentFace/{studentId}")
	public StudentFaceEntity getStudentFace(@PathVariable Long studentId) {

		if (null == studentId) {
			throw new StatusException("B-710001", "studentId is null");
		}

		StudentFaceEntity studentFaceEntity = studentFaceRepo.findOne(studentId);

		if (null == studentFaceEntity) {
			throw new StatusException("B-710001", "studentFaceEntity is null");
		}

		return studentFaceEntity;
	}

}
