package cn.com.qmth.examcloud.core.basic.api.provider;

import java.util.List;

import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.transaction.annotation.Transactional;
import org.springframework.web.bind.annotation.PostMapping;
import org.springframework.web.bind.annotation.RequestBody;
import org.springframework.web.bind.annotation.RequestMapping;
import org.springframework.web.bind.annotation.RestController;

import com.google.common.collect.Lists;

import cn.com.qmth.examcloud.commons.base.exception.StatusException;
import cn.com.qmth.examcloud.commons.web.support.ControllerSupport;
import cn.com.qmth.examcloud.core.basic.api.FaceCloudService;
import cn.com.qmth.examcloud.core.basic.api.bean.FacesetBean;
import cn.com.qmth.examcloud.core.basic.api.bean.StudentFaceBean;
import cn.com.qmth.examcloud.core.basic.api.request.GetStudentFaceReq;
import cn.com.qmth.examcloud.core.basic.api.request.GetUsableFacesetListReq;
import cn.com.qmth.examcloud.core.basic.api.request.SaveStudentFaceReq;
import cn.com.qmth.examcloud.core.basic.api.response.GetStudentFaceResp;
import cn.com.qmth.examcloud.core.basic.api.response.GetUsableFacesetListResp;
import cn.com.qmth.examcloud.core.basic.api.response.SaveStudentFaceResp;
import cn.com.qmth.examcloud.core.basic.dao.FacesetRepo;
import cn.com.qmth.examcloud.core.basic.dao.StudentFaceRepo;
import cn.com.qmth.examcloud.core.basic.dao.StudentRepo;
import cn.com.qmth.examcloud.core.basic.dao.entity.FacesetEntity;
import cn.com.qmth.examcloud.core.basic.dao.entity.StudentFaceEntity;
import cn.com.qmth.examcloud.core.basic.service.FaceService;
import cn.com.qmth.examcloud.core.basic.service.bean.StudentFaceInfo;
import io.swagger.annotations.ApiOperation;

/**
 * 类注释
 *
 * @author WANGWEI
 * @date 2018年8月14日
 * @Copyright (c) 2018-? http://qmth.com.cn All Rights Reserved.
 */
@Transactional
@RestController
@RequestMapping("${$rmp.cloud.basic}" + "studentFace")
public class FaceCloudServiceProvider extends ControllerSupport implements FaceCloudService {

	private static final long serialVersionUID = 8674317301051207328L;

	@Autowired
	StudentFaceRepo studentFaceRepo;

	@Autowired
	FacesetRepo facesetRepo;

	@Autowired
	StudentRepo studentRepo;

	@Autowired
	FaceService studentFaceService;

	@ApiOperation(value = "保存学生人脸数据")
	@PostMapping("saveStudentFace")
	@Override
	public SaveStudentFaceResp saveStudentFace(@RequestBody SaveStudentFaceReq req) {
		trim(req);
		StudentFaceInfo info = new StudentFaceInfo();
		info.setFacesetToken(req.getFacesetToken());
		info.setFaceToken(req.getFaceToken());
		info.setOperator(req.getOperator());
		info.setPhotoName(req.getPhotoName());
		info.setRootOrgId(req.getRootOrgId());
		info.setStudentId(req.getStudentId());
		info.setFaceCount(req.getFaceCount());

		StudentFaceEntity saved = studentFaceService.saveStudentFace(info);
		SaveStudentFaceResp resp = new SaveStudentFaceResp();
		resp.setId(saved.getStudentId());
		return resp;
	}

	@ApiOperation(value = "获取可用的faceset集合")
	@PostMapping("getUsableFacesetList")
	@Override
	public GetUsableFacesetListResp getUsableFacesetList(@RequestBody GetUsableFacesetListReq req) {

		List<FacesetEntity> facesetList = studentFaceService.getUsableFacesetList();

		List<FacesetBean> list = Lists.newArrayList();

		for (FacesetEntity cur : facesetList) {
			FacesetBean bean = new FacesetBean();
			list.add(bean);

			bean.setDisplayName(cur.getDisplayName());
			bean.setFaceCount(cur.getFaceCount());
			bean.setFacesetToken(cur.getFacesetToken());
			bean.setId(cur.getId());
			bean.setOuterId(cur.getOuterId());
			bean.setTags(cur.getTags());
		}

		GetUsableFacesetListResp resp = new GetUsableFacesetListResp();
		resp.setFacesetBeanList(list);
		return resp;
	}

	@ApiOperation(value = "获取可用的faceset集合")
	@PostMapping("getStudentFace")
	@Override
	public GetStudentFaceResp getStudentFace(GetStudentFaceReq req) {
		Long studentId = req.getStudentId();

		if (null == studentId) {
			throw new StatusException("B-710001", "studentId is null");
		}

		StudentFaceEntity studentFaceEntity = studentFaceRepo.findOne(studentId);

		if (null == studentFaceEntity) {
			throw new StatusException("B-710001", "studentFaceEntity is null");
		}

		StudentFaceBean studentFaceBean = new StudentFaceBean();

		studentFaceBean.setCreator(studentFaceEntity.getCreator());
		studentFaceBean.setFacesetId(studentFaceEntity.getFacesetId());
		studentFaceBean.setFaceToken(studentFaceEntity.getFaceToken());
		studentFaceBean.setModifiedBy(studentFaceEntity.getModifiedBy());
		studentFaceBean.setStudentId(studentFaceEntity.getStudentId());

		GetStudentFaceResp resp = new GetStudentFaceResp();
		resp.setStudentFaceBean(studentFaceBean);

		if (null != studentFaceEntity.getFacesetId()) {
			FacesetEntity facesetEntity = facesetRepo.findOne(studentFaceEntity.getFacesetId());
			if (null != facesetEntity) {
				FacesetBean facesetBean = new FacesetBean();
				facesetBean.setDisplayName(facesetEntity.getDisplayName());
				facesetBean.setFaceCount(facesetEntity.getFaceCount());
				facesetBean.setFacesetToken(facesetEntity.getFacesetToken());
				facesetBean.setId(facesetEntity.getId());
				facesetBean.setOuterId(facesetEntity.getOuterId());
				facesetBean.setTags(facesetEntity.getTags());

				resp.setFacesetBean(facesetBean);
			}

		}

		return resp;
	}

}
