package cn.com.qmth.examcloud.core.basic.api.provider;

import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.transaction.annotation.Transactional;
import org.springframework.web.bind.annotation.RequestMapping;
import org.springframework.web.bind.annotation.RestController;

import cn.com.qmth.examcloud.commons.web.support.ControllerSupport;
import cn.com.qmth.examcloud.core.basic.api.RolePrivilegeCloudService;
import cn.com.qmth.examcloud.core.basic.api.bean.RoleBean;
import cn.com.qmth.examcloud.core.basic.api.request.DeleteRoleReq;
import cn.com.qmth.examcloud.core.basic.api.request.SaveRoleReq;
import cn.com.qmth.examcloud.core.basic.api.response.DeleteRoleResp;
import cn.com.qmth.examcloud.core.basic.api.response.SaveRoleResp;
import cn.com.qmth.examcloud.core.basic.dao.PrivilegeGroupRepo;
import cn.com.qmth.examcloud.core.basic.dao.PrivilegeRepo;
import cn.com.qmth.examcloud.core.basic.dao.RolePrivilegeRelationRepo;
import cn.com.qmth.examcloud.core.basic.dao.RoleRepo;
import cn.com.qmth.examcloud.core.basic.dao.entity.RoleEntity;
import cn.com.qmth.examcloud.core.basic.service.RolePrivilegeService;
import cn.com.qmth.examcloud.core.basic.service.bean.RoleInfo;
import io.swagger.annotations.Api;
import io.swagger.annotations.ApiOperation;

/**
 * 角色权限服务
 *
 * @author WANGWEI
 * @date 2018年6月8日
 * @Copyright (c) 2018-? http://qmth.com.cn All Rights Reserved.
 */
@Api("角色权限")
@Transactional
@RestController
@RequestMapping("${$rmp.cloud.basic}" + "rolePrivilege")
public class RolePrivilegeCloudServiceProvider extends ControllerSupport
		implements
			RolePrivilegeCloudService {

	private static final long serialVersionUID = -4360164791713797878L;

	@Autowired
	RoleRepo roleRepo;

	@Autowired
	RolePrivilegeRelationRepo rolePrivilegeRelationRepo;

	@Autowired
	PrivilegeRepo privilegeRepo;

	@Autowired
	PrivilegeGroupRepo privilegeGroupRepo;

	@Autowired
	RolePrivilegeService rolePrivilegeService;

	@Override
	public SaveRoleResp saveRole(SaveRoleReq req) {
		RoleInfo info = new RoleInfo();
		info.setCode(req.getCode());
		info.setName(req.getName());
		info.setRootOrgId(req.getRootOrgId());

		RoleEntity saved = rolePrivilegeService.saveRole(info);

		RoleBean roleBean = new RoleBean();
		roleBean.setRoleCode(saved.getCode());
		roleBean.setRoleId(saved.getId());
		roleBean.setRoleName(saved.getName());

		SaveRoleResp resp = new SaveRoleResp();
		resp.setRoleBean(roleBean);
		return resp;
	}

	@ApiOperation(value = "删除角色")
	@Override
	public DeleteRoleResp deleteRole(DeleteRoleReq req) {
		RoleInfo info = new RoleInfo();
		info.setId(req.getId());
		info.setCode(req.getCode());
		info.setName(req.getName());
		info.setRootOrgId(req.getRootOrgId());

		RoleEntity saved = rolePrivilegeService.deleteRole(info, req.isCascade());

		RoleBean roleBean = new RoleBean();
		roleBean.setRoleCode(saved.getCode());
		roleBean.setRoleId(saved.getId());
		roleBean.setRoleName(saved.getName());

		DeleteRoleResp resp = new DeleteRoleResp();
		resp.setRoleBean(roleBean);
		return resp;
	}

}
