package cn.com.qmth.examcloud.core.basic.api.provider;

import java.util.List;

import org.apache.commons.collections.CollectionUtils;
import org.apache.commons.lang3.StringUtils;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.transaction.annotation.Transactional;
import org.springframework.web.bind.annotation.PostMapping;
import org.springframework.web.bind.annotation.RequestBody;
import org.springframework.web.bind.annotation.RequestMapping;
import org.springframework.web.bind.annotation.RestController;

import com.google.common.collect.Lists;

import cn.com.qmth.examcloud.commons.base.exception.StatusException;
import cn.com.qmth.examcloud.commons.web.security.enums.RoleMeta;
import cn.com.qmth.examcloud.commons.web.support.ControllerSupport;
import cn.com.qmth.examcloud.core.basic.api.UserCloudService;
import cn.com.qmth.examcloud.core.basic.api.bean.RoleBean;
import cn.com.qmth.examcloud.core.basic.api.bean.UserBean;
import cn.com.qmth.examcloud.core.basic.api.request.AddUserReq;
import cn.com.qmth.examcloud.core.basic.api.request.GetUserReq;
import cn.com.qmth.examcloud.core.basic.api.response.AddUserResp;
import cn.com.qmth.examcloud.core.basic.api.response.GetUserResp;
import cn.com.qmth.examcloud.core.basic.base.constants.BasicConsts;
import cn.com.qmth.examcloud.core.basic.dao.OrgRepo;
import cn.com.qmth.examcloud.core.basic.dao.RoleRepo;
import cn.com.qmth.examcloud.core.basic.dao.UserRepo;
import cn.com.qmth.examcloud.core.basic.dao.UserRoleRelationRepo;
import cn.com.qmth.examcloud.core.basic.dao.entity.OrgEntity;
import cn.com.qmth.examcloud.core.basic.dao.entity.RoleEntity;
import cn.com.qmth.examcloud.core.basic.dao.entity.UserEntity;
import cn.com.qmth.examcloud.core.basic.dao.entity.UserRoleRelationEntity;
import io.swagger.annotations.ApiOperation;

/**
 * 类注释
 *
 * @author WANGWEI
 * @date 2018年7月23日
 * @Copyright (c) 2018-? http://qmth.com.cn All Rights Reserved.
 */
@Transactional
@RestController
@RequestMapping("${$rmp.cloud.basic}" + "user")
public class UserCloudServiceProvider extends ControllerSupport implements UserCloudService {

	private static final long serialVersionUID = -7881522922704971610L;

	@Autowired
	UserRepo userRepo;

	@Autowired
	OrgRepo orgRepo;

	@Autowired
	RoleRepo roleRepo;

	@Autowired
	UserRoleRelationRepo userRoleRelationRepo;

	@ApiOperation(value = "保存用户")
	@PostMapping("addUser")
	@Override
	public AddUserResp addUser(@RequestBody AddUserReq req) {
		trim(req);
		Long rootOrgId = req.getRootOrgId();
		String loginName = req.getLoginName();
		String name = req.getName();

		UserEntity userEntity = userRepo.findByRootOrgIdAndLoginName(rootOrgId, loginName);

		if (null != userEntity) {
			throw new StatusException("B-650001", "用户名已经存在");
		}

		userEntity = new UserEntity();
		userEntity.setEnable(true);
		userEntity.setLoginName(loginName);
		userEntity.setRootOrgId(rootOrgId);
		userEntity.setName(name);
		if (StringUtils.isBlank(req.getPassword())) {
			userEntity.setPassword(BasicConsts.DEFAULT_PASSWORD);
		} else {
			userEntity.setPassword(req.getPassword());
		}

		UserEntity saved = userRepo.save(userEntity);

		List<String> roleCodeList = req.getRoleCodeList();

		if (CollectionUtils.isEmpty(roleCodeList)) {
			throw new StatusException("B-650002", "角色不能为空");
		}

		List<UserRoleRelationEntity> userRoles = Lists.newArrayList();
		for (String roleCode : roleCodeList) {
			RoleEntity roleEntity = roleRepo.findByCodeAndRootOrgIdIsNull(roleCode);
			if (null == roleEntity) {
				roleEntity = roleRepo.findByCodeAndRootOrgId(roleCode, rootOrgId);
			}
			if (null == roleEntity) {
				throw new StatusException("B-002002", "role code is wrong. roleCode=" + roleCode);
			}
			if (roleEntity.getCode().equals(RoleMeta.SUPER_ADMIN.name())) {
				throw new StatusException("B-150007", "不允许新增或修改超级管理员");
			}
			UserRoleRelationEntity relation = new UserRoleRelationEntity(saved.getId(),
					roleEntity.getId());
			userRoles.add(relation);
		}

		userRoleRelationRepo.deleteByUserId(saved.getId());
		userRoleRelationRepo.save(userRoles);
		AddUserResp resp = new AddUserResp();
		resp.setUserId(saved.getId());

		return resp;
	}

	@ApiOperation(value = "获取用户")
	@PostMapping("getUser")
	@Override
	public GetUserResp getUser(@RequestBody GetUserReq req) {
		Long rootOrgId = req.getRootOrgId();
		String loginName = req.getLoginName();
		UserEntity userEntity = userRepo.findByRootOrgIdAndLoginName(rootOrgId, loginName);

		GetUserResp resp = new GetUserResp();
		if (null == userEntity) {
			resp.setExisting(false);
			return resp;
		}

		resp.setExisting(true);

		UserBean userBean = new UserBean();
		userBean.setUserId(userEntity.getId());
		userBean.setDisplayName(userEntity.getLoginName() + " (" + userEntity.getName() + ")");
		userBean.setOrgId(userEntity.getOrgId());
		userBean.setRootOrgId(userEntity.getRootOrgId());
		OrgEntity rootOrg = orgRepo.findOne(userBean.getRootOrgId());
		userBean.setRootOrgName(rootOrg.getName());

		if (null != userBean.getOrgId()) {
			OrgEntity org = orgRepo.findOne(userBean.getOrgId());
			userBean.setOrgName(org.getName());
		}

		List<RoleBean> roleList = getUserRoles(userEntity.getId());
		userBean.setRoleList(roleList);

		resp.setUserBean(userBean);
		return resp;
	}

	/**
	 * 获取角色集合
	 *
	 * @author WANGWEI
	 * @param userId
	 * @return
	 */
	private List<RoleBean> getUserRoles(Long userId) {
		List<UserRoleRelationEntity> relationList = userRoleRelationRepo.findAllByUserId(userId);
		List<RoleBean> roleList = Lists.newArrayList();
		if (CollectionUtils.isNotEmpty(relationList)) {
			for (UserRoleRelationEntity cur : relationList) {
				Long roleId = cur.getRoleId();
				RoleEntity roleEntity = roleRepo.findOne(roleId);
				if (null == roleEntity) {
					throw new StatusException("B-002002", "roleId wrong. roleId=" + roleId);
				}
				RoleBean role = new RoleBean(roleEntity.getRootOrgId(), roleEntity.getId(),
						roleEntity.getCode(), roleEntity.getName());
				roleList.add(role);
			}
		}
		return roleList;
	}

}
