package cn.com.qmth.examcloud.core.basic.service.impl;

import org.apache.commons.lang3.StringUtils;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;

import cn.com.qmth.examcloud.commons.base.exception.StatusException;
import cn.com.qmth.examcloud.core.basic.dao.ExamSiteRepo;
import cn.com.qmth.examcloud.core.basic.dao.OrgRepo;
import cn.com.qmth.examcloud.core.basic.dao.entity.ExamSiteEntity;
import cn.com.qmth.examcloud.core.basic.dao.entity.OrgEntity;
import cn.com.qmth.examcloud.core.basic.service.ExamSiteService;
import cn.com.qmth.examcloud.core.basic.service.bean.ExamSiteInfo;

/**
 * 类注释
 *
 * @author WANGWEI
 * @date 2018年8月31日
 * @Copyright (c) 2018-? http://qmth.com.cn All Rights Reserved.
 */
@Service
public class ExamSiteServiceImpl implements ExamSiteService {

	@Autowired
	ExamSiteRepo examSiteRepo;

	@Autowired
	OrgRepo orgRepo;

	@Override
	public ExamSiteEntity saveExamSite(ExamSiteInfo info) {
		Long rootOrgId = info.getRootOrgId();
		Long orgId = info.getOrgId();
		Long id = info.getId();
		String code = info.getCode();
		Boolean enable = info.getEnable();
		String name = info.getName();
		String remark = info.getRemark();
		String contacts = info.getContacts();
		String telephone = info.getTelephone();

		if (null == rootOrgId) {
			throw new StatusException("B-160000", "rootOrgId is null");
		}
		if (null == orgId) {
			throw new StatusException("B-160001", "rootOrgId is null");
		}
		OrgEntity rootOrg = orgRepo.findOne(rootOrgId);
		if (null == rootOrg) {
			throw new StatusException("B-160002", "顶级机构不存在");
		}
		if (null != rootOrg.getParentId()) {
			throw new StatusException("B-160003", "顶级机构错误");
		}

		OrgEntity org = orgRepo.findOne(orgId);
		if (null == org) {
			throw new StatusException("B-160004", "机构不存在");
		}
		if (!org.getParentId().equals(rootOrgId)) {
			throw new StatusException("B-160005", "rootOrgId and orgId is not matched");
		}

		ExamSiteEntity entity = null;

		if (null != id) {
			entity = examSiteRepo.findOne(id);

			if (null == entity) {
				throw new StatusException("B-160006", "考点ID错误");
			}

			if (!rootOrgId.equals(entity.getRootOrgId())) {
				throw new StatusException("B-160007", "顶级机构错误");
			}

		} else if (StringUtils.isNotBlank(code)) {

			entity = examSiteRepo.findByRootOrgIdAndCode(rootOrgId, code);

			if (null != entity) {
				if (null == entity.getEnable()) {
					entity.setEnable(true);
				}
			} else {
				entity = new ExamSiteEntity();
				entity.setRootOrgId(rootOrgId);
				entity.setOrgId(orgId);
				entity.setCode(code);
				entity.setEnable(true);

				if (StringUtils.isBlank(name)) {
					throw new StatusException("B-160008", "name is blank");
				}
			}

		} else {
			throw new StatusException("B-160009", "id,code can not be all null");
		}

		if (null != enable) {
			entity.setEnable(enable);
		}

		if (null != name) {
			if (StringUtils.isBlank(name)) {
				throw new StatusException("B-160008", "name is blank");
			} else {
				entity.setName(name);
			}

		}

		if (null != telephone) {
			entity.setTelephone(telephone);
		}

		if (null != contacts) {
			entity.setContacts(contacts);
		}

		if (null != remark) {
			entity.setRemark(remark);
		}

		ExamSiteEntity saved = examSiteRepo.saveAndFlush(entity);

		return saved;
	}

}
